/*
 / _____)             _              | |
( (____  _____ ____ _| |_ _____  ____| |__
 \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 _____) ) ____| | | || |_| ____( (___| | | |
(______/|_____)_|_|_| \__)_____)\____)_| |_|
    (C)2013 Semtech

Description: SX1276 driver specific target board functions implementation

License: Revised BSD License, see LICENSE.TXT file include in the project

Maintainer: Miguel Luis and Gregory Cristian
*/
#include "radio.h"
#include "sx1276/sx1276.h"
#include "sx1276-board.h"
#include "delay.h"
#include "stdio.h"

//λ
#define SX1278_RST_PIN_INDEX	GPIO_Pin_10
#define SX1278_RST_PIN_GROUP	GPIOB
//SPIʹõspi1(PA5/6/7)ƬѡʹõPA4
#define SX1278_CS_PIN_INDEX	GPIO_Pin_4
#define SX1278_CS_PIN_GROUP	GPIOA
#define SX1278_SCK_PIN_INDEX	GPIO_Pin_5
#define SX1278_SCK_PIN_GROUP	GPIOA
#define SX1278_MISO_PIN_INDEX	GPIO_Pin_6
#define SX1278_MISO_PIN_GROUP	GPIOA
#define SX1278_MOSI_PIN_INDEX	GPIO_Pin_7
#define SX1278_MOSI_PIN_GROUP	GPIOA
//DIO0(޸Ҫֶ޸жϳʼжϺ)
#define SX1278_DIO0_PIN_INDEX	GPIO_Pin_0
#define SX1278_DIO0_PIN_GROUP	GPIOB
//DIO1(޸Ҫֶ޸жϳʼжϺ)
#define SX1278_DIO1_PIN_INDEX	GPIO_Pin_1
#define SX1278_DIO1_PIN_GROUP	GPIOB

/*!
 * Radio driver structure initialization
 */
const struct Radio_s Radio =
{
    SX1276Init,
    SX1276GetStatus,
    SX1276SetModem,
    SX1276SetChannel,
    SX1276IsChannelFree,
    SX1276Random,
    SX1276SetRxConfig,
    SX1276SetTxConfig,
    SX1276CheckRfFrequency,
    SX1276GetTimeOnAir,
    SX1276Send,
    SX1276SetSleep,
    SX1276SetStby, 
    SX1276SetRx,
    SX1276StartCad,
    SX1276ReadRssi,
    SX1276Write,
    SX1276Read,
    SX1276WriteBuffer,
    SX1276ReadBuffer,
    SX1276SetMaxPayloadLength
};

/*!
 * Antenna switch GPIO pins objects
 */
/* ǵRA-01/02ģлӲԶƣҪ
Gpio_t AntSwitchLf;
Gpio_t AntSwitchHf;*/

/*!
 * Tx and Rx timers
 */
/*TimerEvent_t TxTimeoutTimer;
TimerEvent_t RxTimeoutTimer;
TimerEvent_t RxTimeoutSyncWord;*/

//ʵmsʱ
//delayMsʱms
void SX1276DelayMs(uint32_t delayMs){
	delay_ms(delayMs);
}

//ʱʼ(RX,TX,SyncWordʱɺҪ SX1276OnTimeoutIrq() )
void SX1276TimerInit(void){
}

void TIM3_Int_Init(u16 arr,u16 psc)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); //ʱʹ

	TIM3->CR1 |= TIM_CR1_URS;	//ᵼ¶ʱʱж
	
	//ʱTIM3ʼ
	TIM_TimeBaseStructure.TIM_Period = arr; //һ¼װԶװؼĴڵֵ	
	TIM_TimeBaseStructure.TIM_Prescaler =psc; //ΪTIMxʱƵʳԤƵֵ
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1; //ʱӷָ:TDTS = Tck_tim
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  //TIMϼģʽ
	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure); //ָĲʼTIMxʱλ
 
	TIM_ITConfig(TIM3,TIM_IT_Update,ENABLE ); //ʹָTIM3ж,ж
	
	//жȼNVIC
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;  //TIM3ж
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;  //ռȼ0
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;  //ȼ3
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE; //IRQͨʹ
	NVIC_Init(&NVIC_InitStructure);  //ʼNVICĴ
	
	TIM_ClearITPendingBit(TIM3, TIM_IT_Update);

	TIM_Cmd(TIM3, ENABLE);  //TIM					 
}
//ʱ3жϷ
void TIM3_IRQHandler(void)   //TIM3ж
{
	if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET)  //TIM3жϷ
	{
		TIM_ClearITPendingBit(TIM3, TIM_IT_Update  );  //TIMxжϱ־
		TIM_Cmd(TIM3, DISABLE);  //ʧTIMx
		SX1276OnTimeoutIrq();	//жϺҪ SX1276OnTimeoutIrq 
		//printf("\r\ntimer irq\r\n");
	}
}
// tx ʱʱʱtimeoutMs 
void SX1276TxTimeoutTimerStart( uint32_t timeoutMs ){
	//ʹһʱʵʹʹö3ʱʹʱ
	TIM3_Int_Init((timeoutMs * 10) -1,7199);//10KhzļƵʣ10Ϊ1ms 
}

//ر tx ʱʱ
void SX1276TxTimeoutTimerStop(void){
	///ʹһʱʵʹʹö3ʱʹʱ
	TIM_Cmd(TIM3, DISABLE);  //ʧTIMx
}

// RX ʱʱʱtimeoutMs 
void SX1276RxTimeoutTimerStart( uint32_t timeoutMs ){
	//ʹһʱʵʹʹö3ʱʹʱ
	TIM3_Int_Init((timeoutMs * 10) -1,7199);//10KhzļƵʣ10Ϊ1ms 
}

//ر RX ʱʱ
void SX1276RxTimeoutTimerStop(void){
	///ʹһʱʵʹʹö3ʱʹʱ
	TIM_Cmd(TIM3, DISABLE);  //ʧTIMx	
}

// SyncWord ʱʱʱtimeoutMs 
void SX1276SyncWordTimeoutTimerStart( uint32_t timeoutMs ){
	//ʹһʱʵʹʹö3ʱʹʱ
	TIM3_Int_Init((timeoutMs * 10) -1,7199);//10KhzļƵʣ10Ϊ1ms 
}

//ر SyncWord ʱʱ
void SX1276SyncWordTimeoutTimerStop(void){
	///ʹһʱʵʹʹö3ʱʹʱ
	TIM_Cmd(TIM3, DISABLE);  //ʧTIMx	
}

//ʼSPI(csʹܣSPI_CPOL_HighλSPI_CPHA_2EdgeSPI_FirstBit_MSB)Rst
void SX1276IoInit( void )
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	SPI_InitTypeDef  SPI_InitStructure;
	
	//ʼһgpiospi1cs
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB|RCC_APB2Periph_GPIOC, ENABLE);	    //ʹָ˿ʱӣΪ˷ABCȫˣ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Pin = SX1278_CS_PIN_INDEX;
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;	//Ϊ
	GPIO_Init(SX1278_CS_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO

	//SPIĳʼ
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1,ENABLE);//SPIʱʹ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_AF_PP;	//Ƭģʽ
	GPIO_InitStructure.GPIO_Pin = SX1278_SCK_PIN_INDEX|SX1278_MISO_PIN_INDEX|SX1278_MOSI_PIN_INDEX;
	GPIO_Init(SX1278_SCK_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO

	//SPI
	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;  //SPI˫ģʽ:SPIΪ˫˫ȫ˫
	SPI_InitStructure.SPI_Mode = SPI_Mode_Master;		//SPIģʽ:ΪSPI
	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;		//SPIݴС:SPIͽ8λ֡ṹ
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;		//ʱӼ(ͬʱӵĿ״̬Ϊߵƽǵ͵ƽ)
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;	//λ(ͬʱӵĵڼأ½ݱ)
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;		//NSSźӲNSSܽţʹSSIλ:Ϊ(SSI)
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32;		//岨ԤƵֵ:ԤƵֵΪ256(256,֮ٶ,ٶȹ쵼ͨʧٵС)
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;	//ָݴMSBλLSBλʼ:ݴMSBλʼ
	SPI_InitStructure.SPI_CRCPolynomial = 7;	//CRCֵĶʽ(CRCУ)
	SPI_Init(SPI1,&SPI_InitStructure);  //SPI_InitStructָĲʼSPIxĴ
	
	SPI_Cmd(SPI1,ENABLE); //ʹSPI
}

DioIrqHandler **g_irqHandlers;

//ʼDIO()жϣ DIO0~5 GPIOжϺ irqHandlers[0]~irqHandlers[5]
void SX1276IoIrqInit( DioIrqHandler **irqHandlers )
{
	EXTI_InitTypeDef EXTI_InitStructure;
 	NVIC_InitTypeDef NVIC_InitStructure;
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	g_irqHandlers=irqHandlers;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);	//ʹRCC_APB2Periph_AFIOʱ
	
	//ʼDIO0
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB|RCC_APB2Periph_GPIOC, ENABLE);	    //ʹָ˿ʱӣΪ˷ABCȫˣ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IPD;//
	GPIO_InitStructure.GPIO_Pin = SX1278_DIO0_PIN_INDEX;
	GPIO_Init(SX1278_DIO0_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO
	
	//ж
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOB,GPIO_PinSource0);//ӳ߹ϵ( GPIO_PortSourceGPIOX պ0-5ֱpinName/16ʾGPIO_PinSourceXXպ0-15pinName%16ʾ)
	EXTI_InitStructure.EXTI_Line=EXTI_Line0;	//жu8_pinNum
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;//ⲿж
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;//ش
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;//ʹж
	EXTI_Init(&EXTI_InitStructure);//ж

	//жȼ
	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;	//ⲿжXX
	//stm32f103c8ȼӦռȼ2ȼ3
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority=0x02;	//ռȼ
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;   //ȼ
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;	//ʹж
	NVIC_Init(&NVIC_InitStructure);   //ȼ
	
	EXTI_ClearITPendingBit(EXTI_Line0);//жϱ־λ
	
	//DIO1ж
	GPIO_InitStructure.GPIO_Pin = SX1278_DIO1_PIN_INDEX;
	GPIO_Init(SX1278_DIO1_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO
	
	//ж
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOB,GPIO_PinSource1);//ӳ߹ϵ( GPIO_PortSourceGPIOX պ0-5ֱpinName/16ʾGPIO_PinSourceXXպ0-15pinName%16ʾ)
	EXTI_InitStructure.EXTI_Line=EXTI_Line1;	//жu8_pinNum
	EXTI_Init(&EXTI_InitStructure);//ж

	//жȼ
	NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;	//ⲿжXX
	NVIC_Init(&NVIC_InitStructure);   //ȼ
	
	EXTI_ClearITPendingBit(EXTI_Line1);//жϱ־λ
	
	//DIO2~5ʡ
}

//ⲿж0жϴҪصӦĺ
void EXTI0_IRQHandler(void){
	g_irqHandlers[0]();
	EXTI_ClearITPendingBit(EXTI_Line0);//жϱ־λ
}

//ⲿж1жϴҪصӦĺ
void EXTI1_IRQHandler(void){
	g_irqHandlers[1]();
	EXTI_ClearITPendingBit(EXTI_Line1);//жϱ־λ
}

//SPIDIO0~5Rstŵжȥʼ
void SX1276IoDeInit( void )
{
	//ʡȥʼĹ
}

//жƵǷϷ
//Ƶʿֱreturn true
bool SX1276CheckRfFrequency( uint32_t frequency )
{
    // Implement check. Currently all frequencies are supported
    return true;
}

/**
  * Ӳλ
  * rst1msȻ󽫸λΪģʽ
  */
void SX1276Reset( void )
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB|RCC_APB2Periph_GPIOC, ENABLE);	    //ʹָ˿ʱӣΪ˷ABCȫˣ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Pin = SX1278_RST_PIN_INDEX;
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;	//Ϊ
	GPIO_Init(SX1278_RST_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO

	GPIO_ResetBits(SX1278_RST_PIN_GROUP,SX1278_RST_PIN_INDEX);	//͵ƽ
	delay_ms(1);
	
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IN_FLOATING;//
	GPIO_Init(SX1278_RST_PIN_GROUP, &GPIO_InitStructure);	//ʼGPIO
	delay_ms(6);
}

/**
  * SPIƬѡŵĵƽ(͵ƽʹܣߵƽȡ)
  * lev:trueʱߵƽ,falseʱ͵ƽ
  */
void Sx1276SetNSS(bool lev )
{
	if(lev){
		GPIO_SetBits(SX1278_CS_PIN_GROUP,SX1278_CS_PIN_INDEX);	//ߵƽ
	}else{
		GPIO_ResetBits(SX1278_CS_PIN_GROUP,SX1278_CS_PIN_INDEX);	//͵ƽ
	}
}

/**
  * SPIд,dataǷ͵ݣֵǽյ
  */
uint8_t Sx1276SpiInOut(uint16_t data )
{
	//SPIĶд
	uint16_t retry=0;				 	
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET){//ָSPI־λ:ͻձ־λ(˾ͿԿ)
		if((retry++)>2000){
			//ʱ
			return 0xff;
		}
	}
	SPI_I2S_SendData(SPI1, data); //ͨSPIxһ
	retry=0;

	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET){ //ָSPI־λ:ܻǿձ־λǿ˾ͱʾˣ
		if((retry++)>2000){
			//ʱ
			return 0xff;
		}
	}
	return SPI_I2S_ReceiveData(SPI1); //ͨSPIxյ
}
